// Dispense code
// Diyouware TwinTeeth mini-PCB factory,
// Copyright 2015 by V.Aylln.
//

/**
/* write_dispense_header() - writes the gcode header
**/
void write_dispense_header() {
	output(dispense_filename,"wt") {
		printf(
			//"G28 ; home axis\n"
			"M83 ; use relative distances for extrusion\n"
			"G90 ; use absolute coordinates\n"
			"G21 ; set units to millimeters\n"
			"G92 E0 ;reset extruder\n"
			"G0 Z%0.3f F%d\n", DISPENSE_Z_HIGH , DISPENSE_FEED_RATE
			);
	}
}
/**
/* write_dispense_item() - writes a dispense coordinates item
**/
void write_dispense_item(int i) {

	output(dispense_filename,"at") {
		if (tb[i] == NO){
			printf("G0 X%.2f Y%.2f Z%.2f F%d; comp=%s\n", u2mm(x[i]), u2mm(y[i]), DISPENSE_Z_UP, DISPENSE_FEED_RATE, smd_name[i]); // move to new position, above board
			printf("G1 Z%.2f F%d\n", DISPENSE_Z_DOWN, DISPENSE_FEED_RATE_Z);  // ready to dispense.
			printf("G1 E%.5f F%d\n", e[i], DISPENSE_EXTRUDER_FEEDRATE);  // Dispense the paste
			if (DISPENSE_RETRACTION_TIME > 0)
				printf("G1 E-%.5f F%d\n", real((DISPENSE_EXTRUDER_FEEDRATE)/60.0/1000.0*DISPENSE_RETRACTION_TIME), DISPENSE_EXTRUDER_FEEDRATE);  // Retract the paste
			if (DISPENSE_DWELL > 0)
				printf("G4 P%.1f\n",DISPENSE_DWELL);  // Wait given milliseconds; depending on area.
			printf("G1 Z%.2f F%d\n", DISPENSE_Z_UP, DISPENSE_FEED_RATE_Z);  // High up dispenser
		} else {
			printf("; Component %s on bottom layer ignored\n", smd_name[i]); 
		}
	}
}
/**
/* write_dispense_trailer() - writes the end of the gcode file
**/
void write_dispense_trailer(){
	output(dispense_filename,"at") {
	printf("G0 Z%0.3f F%d\n", DISPENSE_Z_HIGH , DISPENSE_FEED_RATE);
	printf("M84 ; shut down motors, done\n");
	}
}

/**
/* dispense_genfiles() - generates the dispensing gcode files
**/
void dispense_genfiles(int n) {
	// Sort data to optimize top/bottom, coord.
	sort(n, index, tb, y, x);  
	// write items to file
	for (int i = 0; i < n; i++) {
		write_dispense_item(index[i]);
	}
}
/**
/* dispense_main() - main solder paste dispensing code
**/
void dispense_main(){

	int n = 0;
	real pad_area = .0;
	real time_ms = .0;

	if (board) {
		dispense_filename = board_name + ".paste.gcode";
		write_dispense_header();
		board(B) {
			// Collect all pad data and store it in memory
			B.elements(E) {
				E.package.contacts(C) {
					if (C.smd) {
						// get SMD-Pad data
						smd_name[n] = E.name;
						pad_area = u2mm(C.smd.dx) * u2mm(C.smd.dy);
						x[n] = C.smd.x;
						y[n] = C.smd.y;
						time_ms = real(DISPENSE_MIN_TIME) + (pad_area * real(DISPENSE_TIME_AREA));

						e[n] = (real(DISPENSE_EXTRUDER_FEEDRATE)/60.0/1000.0) * time_ms;
						tb[n] = E.mirror;
						n++;
					}
				}
			}
			// Gen the files
			dispense_genfiles(n);

		}
		//  Write trailer
		write_dispense_trailer();
	}
}